﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Data;
using Infragistics.Windows.DataPresenter;
using Infragistics.Windows;
using System.Diagnostics;

namespace HierarchicalCheckBoxAll
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        DataSet set;

        public MainWindow()
        {
            InitializeComponent();

            set = GenerateData();
            //RecordSelector r;r.Record.IsSelected
            xamDataGrid1.DataSource = set.Tables["Parent"].DefaultView;
        }

        private DataSet GenerateData()
        {
            var data = new DataSet("SimpleData");
            data.EnforceConstraints = false;

            DataTable parent = new DataTable("Parent");

            parent.Columns.Add("ID", typeof(int));
            parent.Columns.Add("Name", typeof(string));

            parent.PrimaryKey = new[] { parent.Columns[0] };

            DataTable children = new DataTable("Children");

            children.Columns.Add("ID", typeof(int));
            children.Columns.Add("Child", typeof(string));
            children.Columns.Add("Age", typeof(int));
            children.Columns.Add("Parent_ID", typeof(int));

            children.PrimaryKey = new[] { children.Columns[0] };

            data.Tables.Add(parent);
            data.Tables.Add(children);


            data.Relations.Add("Parent_Child", parent.Columns[0], children.Columns[3], false);

            for (int i = 1; i < 4; i++)
            {

                data.Tables["Parent"].Rows.Add(new object[] { i, "Parent" + i });
                for (int j = 0; j < 5; j++)
                {
                    data.Tables["Children"].Rows.Add(new object[] {j, "Child" + j, j + i, i });
                }
            }

            data.AcceptChanges();

            return data;
        }

        private void CheckBox_Loaded(object sender, RoutedEventArgs e)
        {
            DataRecordPresenter r = Utilities.GetAncestorFromType(sender as DependencyObject, typeof(DataRecordPresenter), false) as DataRecordPresenter;
            if (r != null && r.IsHeaderRecord && (r.Record as HeaderRecord).AttachedToRecord.ParentRecord != null)
            {
                HeaderRecord hRecord = r.Record as HeaderRecord;

                if (hRecord.AttachedToRecord.ParentRecord.HasChildren)
                {
                    foreach (Record rec in (hRecord.AttachedToRecord.ParentRecord as ExpandableFieldRecord).ChildRecords)
                    {
                        rec.Tag = sender;
                    }
                }
            }
            else
            {
                foreach (DataRecord rec in r.Record.RecordManager.Current)
                {
                    rec.Tag = sender;
                }
            }
        }

        private void CheckBox_Checked(object sender, RoutedEventArgs e)
        {
            DataRecordPresenter r = Utilities.GetAncestorFromType(sender as DependencyObject, typeof(DataRecordPresenter), false) as DataRecordPresenter;

            SetSelection(r, true);
        }

        private void CheckBox_Unchecked(object sender, RoutedEventArgs e)
        {
            DataRecordPresenter r = Utilities.GetAncestorFromType(sender as DependencyObject, typeof(DataRecordPresenter), false) as DataRecordPresenter;

            SetSelection(r, false);
        }

        void SetSelection(DataRecordPresenter r, bool operation)
        {
            if (r != null && r.IsHeaderRecord && (r.Record as HeaderRecord).AttachedToRecord.ParentRecord != null)
            {
                HeaderRecord hRecord = r.Record as HeaderRecord;

                if (hRecord.AttachedToRecord.ParentRecord.HasChildren)
                {
                    foreach (Record rec in (hRecord.AttachedToRecord.ParentRecord as ExpandableFieldRecord).ChildRecords)
                    {
                        rec.IsSelected = operation;
                    }
                }
            }
            else
            {
                foreach (DataRecord rec in r.Record.RecordManager.Current)
                {
                    rec.IsSelected = operation;
                }
            }
        }

        private void Child_CheckBox_Checked(object sender, RoutedEventArgs e)
        {
            DataRecordPresenter r = Utilities.GetAncestorFromType(sender as DependencyObject, typeof(DataRecordPresenter), false) as DataRecordPresenter;

            if(AreAllRecordsSelectedOrDeselected(r.Record as DataRecord, true) && (r.Record.Tag as CheckBox).IsChecked == false)
            {
                (r.Record.Tag as CheckBox).IsChecked = true;
            }
        }

        private void Child_CheckBox_Unchecked(object sender, RoutedEventArgs e)
        {
            DataRecordPresenter r = Utilities.GetAncestorFromType(sender as DependencyObject, typeof(DataRecordPresenter), false) as DataRecordPresenter;

            if (AreAllRecordsSelectedOrDeselected(r.Record as DataRecord, false) && (r.Record.Tag as CheckBox).IsChecked == true)
            {
                (r.Record.Tag as CheckBox).IsChecked = false;
            }
        }

        public bool AreAllRecordsSelectedOrDeselected(DataRecord r, bool getAllSeleted)
        {
            bool areAllSelected = true;
            if (r.ParentRecord != null && r.ParentRecord is ExpandableFieldRecord)
            {
                foreach (Record rec in (r.ParentRecord as ExpandableFieldRecord).ChildRecords)
                {
                    if (getAllSeleted)
                    {
                        areAllSelected = areAllSelected && rec.IsSelected;
                    }
                    else
                    {
                        areAllSelected = areAllSelected && !rec.IsSelected;
                    }
                }
            }
            else
            {
                foreach (Record rec in r.DataPresenter.Records)
                {
                    if (getAllSeleted)
                    {
                        areAllSelected = areAllSelected && rec.IsSelected;
                    }
                    else
                    {
                        areAllSelected = areAllSelected && !rec.IsSelected;
                    }
                }
            }
            return areAllSelected;
        }

        private void CheckBox_Loaded_1(object sender, RoutedEventArgs e)
        {

        }

    }
}
